@@ -9,7 +9,7 @@ use Sys::Hostname 'hostname';
 
 has 'mysql';
 
-our $VERSION = '0.06';
+our $VERSION = '0.07';
 
 sub dequeue {
   my ($self, $id, $wait, $options) = @_;
@@ -243,6 +243,8 @@ sub _try {
 
   my $tasks = [keys %{$self->minion->tasks}];
 
+  return  unless @$tasks;
+
   my $qq = join(", ", map({ "?" } @{ $options->{queues} // ['default'] }));
   my $qt = join(", ", map({ "?" } @{ $tasks }));
 
@@ -321,11 +323,419 @@ sub receive {
 
 1;
 
-=encoding utf8
+#pod =encoding utf8
+#pod
+#pod =head1 NAME
+#pod
+#pod Minion::Backend::mysql - MySQL backend
+#pod
+#pod =head1 SYNOPSIS
+#pod
+#pod   use Mojolicious::Lite;
+#pod   
+#pod   plugin Minion => {mysql => 'mysql://user@127.0.0.1/minion_jobs'};
+#pod   
+#pod   # Slow task
+#pod   app->minion->add_task(poke_mojo => sub {
+#pod     my $job = shift;
+#pod     $job->app->ua->get('mojolicio.us');
+#pod     $job->app->log->debug('We have poked mojolicio.us for a visitor');
+#pod   });
+#pod   
+#pod   # Perform job in a background worker process
+#pod   get '/' => sub {
+#pod     my $c = shift;
+#pod     $c->minion->enqueue('poke_mojo');
+#pod     $c->render(text => 'We will poke mojolicio.us for you soon.');
+#pod   };
+#pod
+#pod   app->start;
+#pod
+#pod =head1 DESCRIPTION
+#pod
+#pod L<Minion::Backend::mysql> is a backend for L<Minion> based on L<Mojo::mysql>. All
+#pod necessary tables will be created automatically with a set of migrations named
+#pod C<minion>. This backend has been tested on v5.5 of MySQL.
+#pod
+#pod =head1 ATTRIBUTES
+#pod
+#pod L<Minion::Backend::mysql> inherits all attributes from L<Minion::Backend> and
+#pod implements the following new ones.
+#pod
+#pod =head2 mysql
+#pod
+#pod   my $mysql   = $backend->mysql;
+#pod   $backend = $backend->mysql(Mojo::mysql->new);
+#pod
+#pod L<Mojo::mysql> object used to store all data.
+#pod
+#pod =head1 METHODS
+#pod
+#pod L<Minion::Backend::mysql> inherits all methods from L<Minion::Backend> and
+#pod implements the following new ones.
+#pod
+#pod =head2 dequeue
+#pod
+#pod   my $job_info = $backend->dequeue($worker_id, 0.5);
+#pod   my $job_info = $backend->dequeue($worker_id, 0.5, {queues => ['important']});
+#pod
+#pod Wait for job, dequeue it and transition from C<inactive> to C<active> state or
+#pod return C<undef> if queues were empty.
+#pod
+#pod These options are currently available:
+#pod
+#pod =over 2
+#pod
+#pod =item queues
+#pod
+#pod   queues => ['important']
+#pod
+#pod One or more queues to dequeue jobs from, defaults to C<default>.
+#pod
+#pod =back
+#pod
+#pod These fields are currently available:
+#pod
+#pod =over 2
+#pod
+#pod =item args
+#pod
+#pod   args => ['foo', 'bar']
+#pod
+#pod Job arguments.
+#pod
+#pod =item id
+#pod
+#pod   id => '10023'
+#pod
+#pod Job ID.
+#pod
+#pod =item retries
+#pod
+#pod   retries => 3
+#pod
+#pod Number of times job has been retried.
+#pod
+#pod =item task
+#pod
+#pod   task => 'foo'
+#pod
+#pod Task name.
+#pod
+#pod =back
+#pod
+#pod =head2 enqueue
+#pod
+#pod   my $job_id = $backend->enqueue('foo');
+#pod   my $job_id = $backend->enqueue(foo => [@args]);
+#pod   my $job_id = $backend->enqueue(foo => [@args] => {priority => 1});
+#pod
+#pod Enqueue a new job with C<inactive> state.
+#pod
+#pod These options are currently available:
+#pod
+#pod =over 2
+#pod
+#pod =item delay
+#pod
+#pod   delay => 10
+#pod
+#pod Delay job for this many seconds (from now).
+#pod
+#pod =item priority
+#pod
+#pod   priority => 5
+#pod
+#pod Job priority, defaults to C<0>.
+#pod
+#pod =item queue
+#pod
+#pod   queue => 'important'
+#pod
+#pod Queue to put job in, defaults to C<default>.
+#pod
+#pod =back
+#pod
+#pod =head2 fail_job
+#pod
+#pod   my $bool = $backend->fail_job($job_id, $retries);
+#pod   my $bool = $backend->fail_job($job_id, $retries, 'Something went wrong!');
+#pod   my $bool = $backend->fail_job(
+#pod     $job_id, $retries, {msg => 'Something went wrong!'});
+#pod
+#pod Transition from C<active> to C<failed> state.
+#pod
+#pod =head2 finish_job
+#pod
+#pod   my $bool = $backend->finish_job($job_id, $retries);
+#pod   my $bool = $backend->finish_job($job_id, $retries, 'All went well!');
+#pod   my $bool = $backend->finish_job($job_id, $retries, {msg => 'All went well!'});
+#pod
+#pod Transition from C<active> to C<finished> state.
+#pod
+#pod =head2 job_info
+#pod
+#pod   my $job_info = $backend->job_info($job_id);
+#pod
+#pod Get information about a job or return C<undef> if job does not exist.
+#pod
+#pod   # Check job state
+#pod   my $state = $backend->job_info($job_id)->{state};
+#pod
+#pod   # Get job result
+#pod   my $result = $backend->job_info($job_id)->{result};
+#pod
+#pod These fields are currently available:
+#pod
+#pod =over 2
+#pod
+#pod =item args
+#pod
+#pod   args => ['foo', 'bar']
+#pod
+#pod Job arguments.
+#pod
+#pod =item created
+#pod
+#pod   created => 784111777
+#pod
+#pod Time job was created.
+#pod
+#pod =item delayed
+#pod
+#pod   delayed => 784111777
+#pod
+#pod Time job was delayed to.
+#pod
+#pod =item finished
+#pod
+#pod   finished => 784111777
+#pod
+#pod Time job was finished.
+#pod
+#pod =item priority
+#pod
+#pod   priority => 3
+#pod
+#pod Job priority.
+#pod
+#pod =item queue
+#pod
+#pod   queue => 'important'
+#pod
+#pod Queue name.
+#pod
+#pod =item result
+#pod
+#pod   result => 'All went well!'
+#pod
+#pod Job result.
+#pod
+#pod =item retried
+#pod
+#pod   retried => 784111777
+#pod
+#pod Time job has been retried.
+#pod
+#pod =item retries
+#pod
+#pod   retries => 3
+#pod
+#pod Number of times job has been retried.
+#pod
+#pod =item started
+#pod
+#pod   started => 784111777
+#pod
+#pod Time job was started.
+#pod
+#pod =item state
+#pod
+#pod   state => 'inactive'
+#pod
+#pod Current job state, usually C<active>, C<failed>, C<finished> or C<inactive>.
+#pod
+#pod =item task
+#pod
+#pod   task => 'foo'
+#pod
+#pod Task name.
+#pod
+#pod =item worker
+#pod
+#pod   worker => '154'
+#pod
+#pod Id of worker that is processing the job.
+#pod
+#pod =back
+#pod
+#pod =head2 list_jobs
+#pod
+#pod   my $batch = $backend->list_jobs($offset, $limit);
+#pod   my $batch = $backend->list_jobs($offset, $limit, {state => 'inactive'});
+#pod
+#pod Returns the same information as L</"job_info"> but in batches.
+#pod
+#pod These options are currently available:
+#pod
+#pod =over 2
+#pod
+#pod =item state
+#pod
+#pod   state => 'inactive'
+#pod
+#pod List only jobs in this state.
+#pod
+#pod =item task
+#pod
+#pod   task => 'test'
+#pod
+#pod List only jobs for this task.
+#pod
+#pod =back
+#pod
+#pod =head2 list_workers
+#pod
+#pod   my $batch = $backend->list_workers($offset, $limit);
+#pod
+#pod Returns the same information as L</"worker_info"> but in batches.
+#pod
+#pod =head2 new
+#pod
+#pod   my $backend = Minion::Backend::mysql->new('mysql://mysql@/test');
+#pod
+#pod Construct a new L<Minion::Backend::mysql> object.
+#pod
+#pod =head2 register_worker
+#pod
+#pod   my $worker_id = $backend->register_worker;
+#pod   my $worker_id = $backend->register_worker($worker_id);
+#pod
+#pod Register worker or send heartbeat to show that this worker is still alive.
+#pod
+#pod =head2 remove_job
+#pod
+#pod   my $bool = $backend->remove_job($job_id);
+#pod
+#pod Remove C<failed>, C<finished> or C<inactive> job from queue.
+#pod
+#pod =head2 repair
+#pod
+#pod   $backend->repair;
+#pod
+#pod Repair worker registry and job queue if necessary.
+#pod
+#pod =head2 reset
+#pod
+#pod   $backend->reset;
+#pod
+#pod Reset job queue.
+#pod
+#pod =head2 retry_job
+#pod
+#pod   my $bool = $backend->retry_job($job_id, $retries);
+#pod   my $bool = $backend->retry_job($job_id, $retries, {delay => 10});
+#pod
+#pod Transition from C<failed> or C<finished> state back to C<inactive>.
+#pod
+#pod These options are currently available:
+#pod
+#pod =over 2
+#pod
+#pod =item delay
+#pod
+#pod   delay => 10
+#pod
+#pod Delay job for this many seconds (from now).
+#pod
+#pod =item priority
+#pod
+#pod   priority => 5
+#pod
+#pod Job priority.
+#pod
+#pod =item queue
+#pod
+#pod   queue => 'important'
+#pod
+#pod Queue to put job in.
+#pod
+#pod =back
+#pod
+#pod =head2 stats
+#pod
+#pod   my $stats = $backend->stats;
+#pod
+#pod Get statistics for jobs and workers.
+#pod
+#pod =head2 unregister_worker
+#pod
+#pod   $backend->unregister_worker($worker_id);
+#pod
+#pod Unregister worker.
+#pod
+#pod =head2 worker_info
+#pod
+#pod   my $worker_info = $backend->worker_info($worker_id);
+#pod
+#pod Get information about a worker or return C<undef> if worker does not exist.
+#pod
+#pod   # Check worker host
+#pod   my $host = $backend->worker_info($worker_id)->{host};
+#pod
+#pod These fields are currently available:
+#pod
+#pod =over 2
+#pod
+#pod =item host
+#pod
+#pod   host => 'localhost'
+#pod
+#pod Worker host.
+#pod
+#pod =item jobs
+#pod
+#pod   jobs => ['10023', '10024', '10025', '10029']
+#pod
+#pod Ids of jobs the worker is currently processing.
+#pod
+#pod =item notified
+#pod
+#pod   notified => 784111777
+#pod
+#pod Last time worker sent a heartbeat.
+#pod
+#pod =item pid
+#pod
+#pod   pid => 12345
+#pod
+#pod Process id of worker.
+#pod
+#pod =item started
+#pod
+#pod   started => 784111777
+#pod
+#pod Time worker was started.
+#pod
+#pod =back
+#pod
+#pod =head1 SEE ALSO
+#pod
+#pod L<Minion>, L<Mojolicious::Guides>, L<http://mojolicio.us>.
+#pod
+#pod =cut
+
+=pod
+
+=encoding UTF-8
 
 =head1 NAME
 
-Minion::Backend::mysql - MySQL backend
+Minion::Backend::mysql
+
+=head1 VERSION
+
+version 0.07
 
 =head1 SYNOPSIS
 
@@ -355,6 +765,10 @@ L<Minion::Backend::mysql> is a backend for L<Minion> based on L<Mojo::mysql>. Al
 necessary tables will be created automatically with a set of migrations named
 C<minion>. This backend has been tested on v5.5 of MySQL.
 
+=head1 NAME
+
+Minion::Backend::mysql - MySQL backend
+
 =head1 ATTRIBUTES
 
 L<Minion::Backend::mysql> inherits all attributes from L<Minion::Backend> and
@@ -721,6 +1135,59 @@ Time worker was started.
 
 L<Minion>, L<Mojolicious::Guides>, L<http://mojolicio.us>.
 
+=head1 AUTHORS
+
+=over 4
+
+=item *
+
+Brian Medley <bpmedley@cpan.org>
+
+=item *
+
+Doug Bell <preaction@cpan.org>
+
+=back
+
+=head1 CONTRIBUTORS
+
+=for stopwords Alexander Nalobin Brian Medley Doug Bell Olaf Alders Zoffix Znet
+
+=over 4
+
+=item *
+
+Alexander Nalobin <nalobin@reg.ru>
+
+=item *
+
+Brian Medley <bpm@pxe-server.scnet.net>
+
+=item *
+
+Brian Medley <pub-github@bmedley.org>
+
+=item *
+
+Doug Bell <doug@preaction.me>
+
+=item *
+
+Olaf Alders <olaf@wundersolutions.com>
+
+=item *
+
+Zoffix Znet <cpan@zoffix.com>
+
+=back
+
+=head1 COPYRIGHT AND LICENSE
+
+This software is copyright (c) 2017 by Doug Bell and Brian Medley.
+
+This is free software; you can redistribute it and/or modify it under
+the same terms as the Perl 5 programming language system itself.
+
 =cut
 
 __DATA__
@@ -729,12 +1196,12 @@ __DATA__
 -- 1 up
 create table if not exists minion_jobs (
 		`id`       serial not null primary key,
-		`args`     text not null,
+		`args`     mediumblob not null,
 		`created`  timestamp not null default current_timestamp,
 		`delayed`  timestamp not null default current_timestamp,
 		`finished` timestamp null,
 		`priority` int not null,
-		`result`   text,
+		`result`   mediumblob,
 		`retried`  timestamp null,
 		`retries`  int not null default 0,
 		`started`  timestamp null,
@@ -770,3 +1237,8 @@ CREATE TABLE IF NOT EXISTS minion_workers_inbox (
   `message` BLOB NOT NULL
 );
 ALTER TABLE minion_jobs ADD COLUMN attempts INT NOT NULL DEFAULT 1;
+
+-- 5 up
+ALTER TABLE minion_jobs MODIFY COLUMN args MEDIUMBLOB NOT NULL;
+ALTER TABLE minion_jobs MODIFY COLUMN result MEDIUMBLOB;
+
